@@ -126,7 +126,7 @@ PGAPI_Connect(
 	/* fill in any defaults */
 	getDSNdefaults(ci);
 
-	qlog("conn = %u, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password);
+	qlog("conn = %u, %s(DSN='%s', UID='%s', PWD='%s')\n", conn, func, ci->dsn, ci->username, ci->password ? "xxxxx" : "");
 
 	if (CC_connect(conn, AUTH_REQ_OK, NULL) <= 0)
 	{
@@ -238,6 +238,8 @@ CC_conninfo_init(ConnInfo *conninfo)
 		conninfo->lf_conversion = -1;
 		conninfo->true_is_minus1 = -1;
 		conninfo->int8_as = -101;
+		conninfo->bytea_as_longvarbinary = -1;
+		conninfo->use_server_side_prepare = -1;
 		memcpy(&(conninfo->drivers), &globals, sizeof(globals));
 }
 /*
@@ -273,7 +275,7 @@ CC_Constructor()
 
 		rv->num_stmts = STMT_INCREMENT;
 
-		rv->lobj_type = PG_TYPE_LO;
+		rv->lobj_type = PG_TYPE_LO_UNDEFINED;
 
 		rv->ntables = 0;
 		rv->col_info = NULL;
@@ -459,13 +461,12 @@ CC_cleanup(ConnectionClass *self)
 	/* We are always in the middle of a transaction, */
 	/* even if we are in auto commit. */
 	if (self->sock)
+	{
 		CC_abort(self);
 
-	mylog("after CC_abort\n");
+		mylog("after CC_abort\n");
 
-	/* This actually closes the connection to the dbase */
-	if (self->sock)
-	{
+		/* This actually closes the connection to the dbase */
 		SOCK_Destructor(self->sock);
 		self->sock = NULL;
 	}
@@ -499,14 +500,20 @@ CC_cleanup(ConnectionClass *self)
 	self->transact_status = CONN_IN_AUTOCOMMIT;
 	CC_conninfo_init(&(self->connInfo));
 	if (self->client_encoding)
+	{
 		free(self->client_encoding);
-	self->client_encoding = NULL;
+		self->client_encoding = NULL;
+	}
 	if (self->server_encoding)
+	{
 		free(self->server_encoding);
-	self->server_encoding = NULL;
+		self->server_encoding = NULL;
+	}
 	if (self->current_schema)
+	{
 		free(self->current_schema);
-	self->current_schema = NULL;
+		self->current_schema = NULL;
+	}
 	/* Free cached table info */
 	if (self->col_info)
 	{
@@ -673,7 +680,7 @@ CC_connect(ConnectionClass *self, char password_req, char *salt_para)
 			return 0;
 		}
 
-		mylog("CC_connect(): DSN = '%s', server = '%s', port = '%s', database = '%s', username = '%s', password='%s'\n", ci->dsn, ci->server, ci->port, ci->database, ci->username, ci->password);
+		mylog("CC_connect(): DSN = '%s', server = '%s', port = '%s', database = '%s', username = '%s', password='%s'\n", ci->dsn, ci->server, ci->port, ci->database, ci->username, ci->password ? "xxxxx" : "");
 
 another_version_retry:
 
@@ -1825,6 +1832,17 @@ CC_send_settings(ConnectionClass *self)
 
 	}
 
+	/* extra_float_digits (applicable since 7.4) */
+	if (PG_VERSION_GT(self, 7.3))
+	{
+		result = PGAPI_ExecDirect(hstmt, "set extra_float_digits to 2", SQL_NTS);
+		if ((result != SQL_SUCCESS) && (result != SQL_SUCCESS_WITH_INFO))
+			status = FALSE;
+
+		mylog("%s: result %d, status %d from set extra_float_digits\n", func, result, status);
+
+	}
+
 	/* Global settings */
 	if (ci->drivers.conn_settings[0] != '\0')
 	{
