@@ -87,7 +87,7 @@ typedef enum
 #define CC_set_errornumber(x, n)	(x->__error_number = n)
 
 /* For Multi-thread */
-#if defined(WIN_MULTITHREAD_SUPPORT)
+#if defined(WIN_FREETHREAD_SUPPORT)
 #define INIT_CONN_CS(x)		InitializeCriticalSection(&((x)->cs))
 #define ENTER_CONN_CS(x)	EnterCriticalSection(&((x)->cs))
 #define LEAVE_CONN_CS(x)	LeaveCriticalSection(&((x)->cs))
@@ -179,7 +179,7 @@ typedef struct
 {
 	char		dsn[MEDIUM_REGISTRY_LEN];
 	char		desc[MEDIUM_REGISTRY_LEN];
-	char		driver[MEDIUM_REGISTRY_LEN];
+	char		drivername[MEDIUM_REGISTRY_LEN];
 	char		server[MEDIUM_REGISTRY_LEN];
 	char		database[MEDIUM_REGISTRY_LEN];
 	char		username[MEDIUM_REGISTRY_LEN];
@@ -195,12 +195,14 @@ typedef struct
 	char		translation_dll[MEDIUM_REGISTRY_LEN];
 	char		translation_option[SMALL_REGISTRY_LEN];
 	char		focus_password;
-	char		disallow_premature;
-	char		allow_keyset;
-	char		updatable_cursors;
-	char		lf_conversion;
-	char		true_is_minus1;
-	char		int8_as;
+	signed char	disallow_premature;
+	signed char	allow_keyset;
+	signed char	updatable_cursors;
+	signed char	lf_conversion;
+	signed char	true_is_minus1;
+	signed char	int8_as;
+	signed char	bytea_as_longvarbinary;
+	signed char	use_server_side_prepare;
 	GLOBAL_VALUES drivers;		/* moved from driver's option */
 } ConnInfo;
 
