@@ -39,6 +39,10 @@
 #include "connection.h"
 #include "pgapifunc.h"
 
+#if defined(UNICODE_SUPPORT) && defined(WIN32)
+#define	WIN_UNICODE_SUPPORT
+#endif
+
 #ifdef	__CYGWIN__
 #define TIMEZONE_GLOBAL _timezone
 #elif	defined(WIN32) || defined(HAVE_INT_TIMEZONE)
@@ -131,6 +135,7 @@ static const char *mapFunction(const char *func, int param_count);
 static unsigned int conv_from_octal(const unsigned char *s);
 static unsigned int conv_from_hex(const unsigned char *s);
 static char *conv_to_octal(unsigned char val, char *octal);
+static int pg_bin2hex(UCHAR *src, UCHAR *dst, int length);
 
 /*---------
  *			A Guide for date/time/timestamp conversions
@@ -151,6 +156,23 @@ static char *conv_to_octal(unsigned char val, char *octal);
  */
 
 
+/*
+ *	Macros for BIGINT handling.
+ */
+#ifdef	ODBCINT64
+#ifdef	WIN32
+#define	ATOI64	_atoi64
+#define	ATOI64U	_atoi64
+#define	FORMATI64	"%I64d"
+#define	FORMATI64U	"%I64u"
+#elif	defined(HAVE_STRTOLL)
+#define	ATOI64(val)	strtoll(val, NULL, 10)
+#define	ATOI64U(val)	strtoull(val, NULL, 10)
+#define	FORMATI64	"%lld"
+#define	FORMATI64U	"%llu"
+#else /* HAVE_STRTOLL */
+#endif /* WIN32 */
+#endif /* ODBCINT64 */
 
 /*
  *	TIMESTAMP <-----> SIMPLE_TIME
@@ -347,8 +369,8 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 	ARDFields	*opts = SC_get_ARD(stmt);
 	Int4		len = 0,
 				copy_len = 0;
-	SIMPLE_TIME st;
-	time_t		t = time(NULL);
+	SIMPLE_TIME std_time;
+	time_t		stmt_t = SC_get_time(stmt);
 	struct tm  *tim;
 #ifdef	HAVE_LOCALTIME_R
 	struct tm  tm;
@@ -360,6 +382,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 	int			bind_row = stmt->bind_row;
 	int			bind_size = opts->bind_size;
 	int			result = COPY_OK;
+	ConnectionClass		*conn = SC_get_conn(stmt);
 #ifdef HAVE_LOCALE_H
 	char saved_locale[256];
 #endif /* HAVE_LOCALE_H */
@@ -372,6 +395,10 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 #ifdef	UNICODE_SUPPORT
 	BOOL	wchanged =   FALSE;
 #endif /* UNICODE_SUPPORT */
+#ifdef	WIN_UNICODE_SUPPORT
+	SQLWCHAR	*allocbuf = NULL;
+	Int4		wstrlen;	
+#endif /* WIN_UNICODE_SUPPORT */
 
 	if (stmt->current_col >= 0)
 	{
@@ -406,17 +433,17 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 
 	}
 
-	memset(&st, 0, sizeof(SIMPLE_TIME));
+	memset(&std_time, 0, sizeof(SIMPLE_TIME));
 
 	/* Initialize current date */
 #ifdef	HAVE_LOCALTIME_R
-	tim = localtime_r(&t, &tm);
+	tim = localtime_r(&stmt_t, &tm);
 #else
-	tim = localtime(&t);
+	tim = localtime(&stmt_t);
 #endif /* HAVE_LOCALTIME_R */
-	st.m = tim->tm_mon + 1;
-	st.d = tim->tm_mday;
-	st.y = tim->tm_year + 1900;
+	std_time.m = tim->tm_mon + 1;
+	std_time.d = tim->tm_mday;
+	std_time.y = tim->tm_year + 1900;
 
 	mylog("copy_and_convert: field_type = %d, fctype = %d, value = '%s', cbValueMax=%d\n", field_type, fCType, (value == NULL) ? "<NULL>" : value, cbValueMax);
 
@@ -463,50 +490,51 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 			 * PG_TYPE_CHAR,VARCHAR $$$
 			 */
 		case PG_TYPE_DATE:
-			sscanf(value, "%4d-%2d-%2d", &st.y, &st.m, &st.d);
+			sscanf(value, "%4d-%2d-%2d", &std_time.y, &std_time.m, &std_time.d);
 			break;
 
 		case PG_TYPE_TIME:
-			sscanf(value, "%2d:%2d:%2d", &st.hh, &st.mm, &st.ss);
+			sscanf(value, "%2d:%2d:%2d", &std_time.hh, &std_time.mm, &std_time.ss);
 			break;
 
 		case PG_TYPE_ABSTIME:
 		case PG_TYPE_DATETIME:
 		case PG_TYPE_TIMESTAMP_NO_TMZONE:
 		case PG_TYPE_TIMESTAMP:
-			st.fr = 0;
-			st.infinity = 0;
+			std_time.fr = 0;
+			std_time.infinity = 0;
 			if (strnicmp(value, "infinity", 8) == 0)
 			{
-				st.infinity = 1;
-				st.m = 12;
-				st.d = 31;
-				st.y = 9999;
-				st.hh = 23;
-				st.mm = 59;
-				st.ss = 59;
+				std_time.infinity = 1;
+				std_time.m = 12;
+				std_time.d = 31;
+				std_time.y = 9999;
+				std_time.hh = 23;
+				std_time.mm = 59;
+				std_time.ss = 59;
 			}
 			if (strnicmp(value, "-infinity", 9) == 0)
 			{
-				st.infinity = -1;
-				st.m = 0;
-				st.d = 0;
-				st.y = 0;
-				st.hh = 0;
-				st.mm = 0;
-				st.ss = 0;
+				std_time.infinity = -1;
+				std_time.m = 0;
+				std_time.d = 0;
+				std_time.y = 0;
+				std_time.hh = 0;
+				std_time.mm = 0;
+				std_time.ss = 0;
 			}
 			if (strnicmp(value, "invalid", 7) != 0)
 			{
-				BOOL		bZone = (field_type != PG_TYPE_TIMESTAMP_NO_TMZONE && PG_VERSION_GE(SC_get_conn(stmt), 7.2));
+				BOOL		bZone = (field_type != PG_TYPE_TIMESTAMP_NO_TMZONE && PG_VERSION_GE(conn, 7.2));
 				int			zone;
 
 				/*
-				 * sscanf(value, "%4d-%2d-%2d %2d:%2d:%2d", &st.y, &st.m,
-				 * &st.d, &st.hh, &st.mm, &st.ss);
+				 * sscanf(value, "%4d-%2d-%2d %2d:%2d:%2d", &std_time.y, &std_time.m,
+				 * &std_time.d, &std_time.hh, &std_time.mm, &std_time.ss);
 				 */
 				bZone = FALSE;	/* time zone stuff is unreliable */
-				timestamp2stime(value, &st, &bZone, &zone);
+				timestamp2stime(value, &std_time, &bZone, &zone);
+inolog("2stime fr=%d\n", std_time.fr);
 			}
 			else
 			{
@@ -514,18 +542,18 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				 * The timestamp is invalid so set something conspicuous,
 				 * like the epoch
 				 */
-				t = 0;
+				time_t	t = 0;
 #ifdef	HAVE_LOCALTIME_R
 				tim = localtime_r(&t, &tm);
 #else
 				tim = localtime(&t);
 #endif /* HAVE_LOCALTIME_R */
-				st.m = tim->tm_mon + 1;
-				st.d = tim->tm_mday;
-				st.y = tim->tm_year + 1900;
-				st.hh = tim->tm_hour;
-				st.mm = tim->tm_min;
-				st.ss = tim->tm_sec;
+				std_time.m = tim->tm_mon + 1;
+				std_time.d = tim->tm_mday;
+				std_time.y = tim->tm_year + 1900;
+				std_time.hh = tim->tm_hour;
+				std_time.mm = tim->tm_min;
+				std_time.ss = tim->tm_sec;
 			}
 			break;
 
@@ -614,7 +642,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 			 * This is a large object OID, which is used to store
 			 * LONGVARBINARY objects.
 			 */
-		case PG_TYPE_LO:
+		case PG_TYPE_LO_UNDEFINED:
 
 			return convert_lo(stmt, value, fCType, ((char *) rgbValue + rgbValueOffset), cbValueMax, (SDWORD *) ((char *) pcbValue + pcbValueOffset));
 
@@ -636,10 +664,11 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 	rgbValueBindRow = (char *) rgbValue + rgbValueOffset;
 
 #ifdef	UNICODE_SUPPORT
-	if (fCType == SQL_C_CHAR || fCType == SQL_C_WCHAR)
+	if (fCType == SQL_C_CHAR || fCType == SQL_C_WCHAR
 #else
-	if (fCType == SQL_C_CHAR)
+	if (fCType == SQL_C_CHAR
 #endif /* UNICODE_SUPPORT */
+	    || fCType == INTERNAL_ASIS_TYPE)
 	{
 		/* Special character formatting as required */
 
@@ -652,13 +681,13 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 			case PG_TYPE_DATE:
 				len = 10;
 				if (cbValueMax > len)
-					sprintf(rgbValueBindRow, "%.4d-%.2d-%.2d", st.y, st.m, st.d);
+					sprintf(rgbValueBindRow, "%.4d-%.2d-%.2d", std_time.y, std_time.m, std_time.d);
 				break;
 
 			case PG_TYPE_TIME:
 				len = 8;
 				if (cbValueMax > len)
-					sprintf(rgbValueBindRow, "%.2d:%.2d:%.2d", st.hh, st.mm, st.ss);
+					sprintf(rgbValueBindRow, "%.2d:%.2d:%.2d", std_time.hh, std_time.mm, std_time.ss);
 				break;
 
 			case PG_TYPE_ABSTIME:
@@ -668,7 +697,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				len = 19;
 				if (cbValueMax > len)
 					sprintf(rgbValueBindRow, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d",
-							st.y, st.m, st.d, st.hh, st.mm, st.ss);
+							std_time.y, std_time.m, std_time.d, std_time.hh, std_time.mm, std_time.ss);
 				break;
 
 			case PG_TYPE_BOOL:
@@ -695,10 +724,6 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				 */
 			case PG_TYPE_BYTEA:/* convert binary data to hex strings
 								 * (i.e, 255 = "FF") */
-				len = convert_pgbinary_to_char(neut_str, rgbValueBindRow, cbValueMax);
-
-				/***** THIS IS NOT PROPERLY IMPLEMENTED *****/
-				break;
 
 			default:
 				if (stmt->current_col < 0)
@@ -710,7 +735,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 					pbic = &opts->bindings[stmt->current_col];
 				if (pbic->data_left < 0)
 				{
-					BOOL lf_conv = SC_get_conn(stmt)->connInfo.lf_conversion;
+					BOOL lf_conv = conn->connInfo.lf_conversion;
 #ifdef	UNICODE_SUPPORT
 					if (fCType == SQL_C_WCHAR)
 					{
@@ -720,12 +745,34 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 					}
 					else
 #endif /* UNICODE_SUPPORT */
-					/* convert linefeeds to carriage-return/linefeed */
-					len = convert_linefeeds(neut_str, NULL, 0, lf_conv, &changed);
+					if (PG_TYPE_BYTEA == field_type)
+					{
+						len = convert_from_pgbinary(neut_str, NULL, 0);
+						len *= 2;
+						changed = TRUE;
+					}
+					else
+#ifdef	WIN_UNICODE_SUPPORT
+					if (fCType == SQL_C_CHAR)
+					{
+						wstrlen = utf8_to_ucs2_lf(neut_str, -1, lf_conv, NULL, 0);
+						allocbuf = (SQLWCHAR *) malloc(2 * (wstrlen + 1));
+						wstrlen = utf8_to_ucs2_lf(neut_str, -1, lf_conv, allocbuf, wstrlen + 1);
+						len = WideCharToMultiByte(CP_ACP, 0, (LPCWSTR) allocbuf, wstrlen, NULL, 0, NULL, NULL);
+						changed = TRUE;
+					}
+					else
+#endif /* WIN_UNICODE_SUPPORT */
+						/* convert linefeeds to carriage-return/linefeed */
+						len = convert_linefeeds(neut_str, NULL, 0, lf_conv, &changed);
 					if (cbValueMax == 0)		/* just returns length
 												 * info */
 					{
 						result = COPY_RESULT_TRUNCATED;
+#ifdef	WIN_UNICODE_SUPPORT
+						if (allocbuf)
+							free(allocbuf);
+#endif /* WIN_UNICODE_SUPPORT */
 						break;
 					}
 					if (!pbic->ttlbuf)
@@ -744,8 +791,25 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 						}
 						else
 #endif /* UNICODE_SUPPORT */
-						convert_linefeeds(neut_str, pbic->ttlbuf, pbic->ttlbuflen, lf_conv, &changed);
+						if (PG_TYPE_BYTEA == field_type)
+						{
+							len = convert_from_pgbinary(neut_str, pbic->ttlbuf, pbic->ttlbuflen);
+							pg_bin2hex(pbic->ttlbuf, pbic->ttlbuf, len);
+							len *= 2; 
+						}
+						else
+#ifdef	WIN_UNICODE_SUPPORT
+						if (fCType == SQL_C_CHAR)
+						{
+							len = WideCharToMultiByte(CP_ACP, 0, allocbuf, wstrlen, pbic->ttlbuf, pbic->ttlbuflen, NULL, NULL);
+							free(allocbuf);
+							allocbuf = NULL;
+						}
+						else
+#endif /* WIN_UNICODE_SUPPORT */
+							convert_linefeeds(neut_str, pbic->ttlbuf, pbic->ttlbuflen, lf_conv, &changed);
 						ptr = pbic->ttlbuf;
+						pbic->ttlbufused = len;
 					}
 					else
 					{
@@ -758,7 +822,10 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 					}
 				}
 				else
+				{
 					ptr = pbic->ttlbuf;
+					len = pbic->ttlbufused;
+				}
 
 				mylog("DEFAULT: len = %d, ptr = '%s'\n", len, ptr);
 
@@ -766,7 +833,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				{
 					if (pbic->data_left > 0)
 					{
-						ptr += strlen(ptr) - pbic->data_left;
+						ptr += len - pbic->data_left;
 						len = pbic->data_left;
 					}
 					else
@@ -890,9 +957,9 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 						ds = (DATE_STRUCT *) ((char *) rgbValue + (bind_row * bind_size));
 					else
 						ds = (DATE_STRUCT *) rgbValue + bind_row;
-					ds->year = st.y;
-					ds->month = st.m;
-					ds->day = st.d;
+					ds->year = std_time.y;
+					ds->month = std_time.m;
+					ds->day = std_time.d;
 				}
 				break;
 
@@ -908,9 +975,9 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 						ts = (TIME_STRUCT *) ((char *) rgbValue + (bind_row * bind_size));
 					else
 						ts = (TIME_STRUCT *) rgbValue + bind_row;
-					ts->hour = st.hh;
-					ts->minute = st.mm;
-					ts->second = st.ss;
+					ts->hour = std_time.hh;
+					ts->minute = std_time.mm;
+					ts->second = std_time.ss;
 				}
 				break;
 
@@ -926,13 +993,13 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 						ts = (TIMESTAMP_STRUCT *) ((char *) rgbValue + (bind_row * bind_size));
 					else
 						ts = (TIMESTAMP_STRUCT *) rgbValue + bind_row;
-					ts->year = st.y;
-					ts->month = st.m;
-					ts->day = st.d;
-					ts->hour = st.hh;
-					ts->minute = st.mm;
-					ts->second = st.ss;
-					ts->fraction = st.fr;
+					ts->year = std_time.y;
+					ts->month = std_time.m;
+					ts->day = std_time.d;
+					ts->hour = std_time.hh;
+					ts->minute = std_time.mm;
+					ts->second = std_time.ss;
+					ts->fraction = std_time.fr;
 				}
 				break;
 
@@ -1117,27 +1184,36 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				break;
 
 #if (ODBCVER >= 0x0300) && defined(ODBCINT64)
-#ifdef WIN32
 			case SQL_C_SBIGINT:
 				len = 8;
 				if (bind_size > 0)
-					*(SQLBIGINT *) ((char *) rgbValue + (bind_row * bind_size)) = _atoi64(neut_str);
+					*(SQLBIGINT *) ((char *) rgbValue + (bind_row * bind_size)) = ATOI64(neut_str);
 				else
-					*((SQLBIGINT *) rgbValue + bind_row) = _atoi64(neut_str);
+					*((SQLBIGINT *) rgbValue + bind_row) = ATOI64(neut_str);
 				break;
 
 			case SQL_C_UBIGINT:
 				len = 8;
 				if (bind_size > 0)
-					*(SQLUBIGINT *) ((char *) rgbValue + (bind_row * bind_size)) = _atoi64(neut_str);
+					*(SQLUBIGINT *) ((char *) rgbValue + (bind_row * bind_size)) = ATOI64U(neut_str);
 				else
-					*((SQLUBIGINT *) rgbValue + bind_row) = _atoi64(neut_str);
+					*((SQLUBIGINT *) rgbValue + bind_row) = ATOI64U(neut_str);
 				break;
 
-#endif /* WIN32 */
 #endif /* ODBCINT64 */
 			case SQL_C_BINARY:
 
+				if (PG_TYPE_UNKNOWN == field_type)
+				{
+					if (pcbValue)
+						*(SDWORD *) ((char *) pcbValue + pcbValueOffset) = SQL_NULL_DATA;
+					return COPY_OK;
+				}
+				else if (PG_TYPE_BYTEA != field_type)
+				{
+					mylog("couldn't convert the type %d to SQL_C_BINARY\n", field_type);
+					return COPY_UNSUPPORTED_TYPE;
+				}
 				/* truncate if necessary */
 				/* convert octal escapes to bytes */
 
@@ -1150,12 +1226,24 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 					pbic = &opts->bindings[stmt->current_col];
 				if (!pbic->ttlbuf)
 					pbic->ttlbuflen = 0;
-				if (len = strlen(neut_str), len >= (int) pbic->ttlbuflen)
+				if (pbic->data_left < 0)
 				{
-					pbic->ttlbuf = realloc(pbic->ttlbuf, len + 1);
-					pbic->ttlbuflen = len + 1;
+					if (cbValueMax <= 0)
+					{
+						len = convert_from_pgbinary(neut_str, NULL, 0);
+						result = COPY_RESULT_TRUNCATED;
+						break;
+					}
+					if (len = strlen(neut_str), len >= (int) pbic->ttlbuflen)
+					{
+						pbic->ttlbuf = realloc(pbic->ttlbuf, len + 1);
+						pbic->ttlbuflen = len + 1;
+					}
+					len = convert_from_pgbinary(neut_str, pbic->ttlbuf, pbic->ttlbuflen);
+					pbic->ttlbufused = len;
 				}
-				len = convert_from_pgbinary(neut_str, pbic->ttlbuf, pbic->ttlbuflen);
+				else
+					len = pbic->ttlbufused;
 				ptr = pbic->ttlbuf;
 
 				if (stmt->current_col >= 0)
@@ -1194,8 +1282,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 				 */
 				if (len > cbValueMax)
 					result = COPY_RESULT_TRUNCATED;
-
-				if (pbic->ttlbuf)
+				else if (pbic->ttlbuf)
 				{
 					free(pbic->ttlbuf);
 					pbic->ttlbuf = NULL;
@@ -1232,6 +1319,7 @@ copy_and_convert_field(StatementClass *stmt, Int4 field_type, void *value, Int2
 #define	FLGP_CURSOR_CHECK_OK	(1L << 1)
 #define	FLGP_SELECT_INTO		(1L << 2)
 #define	FLGP_SELECT_FOR_UPDATE	(1L << 3)
+#define	FLGP_BUILDING_PREPARE_STATEMENT	(1L << 4)
 typedef struct _QueryParse {
 	const char	*statement;
 	int		statement_type;
@@ -1252,7 +1340,7 @@ typedef struct _QueryParse {
 static int
 QP_initialize(QueryParse *q, const StatementClass *stmt)
 {
-	q->statement = stmt->statement;
+	q->statement = stmt->execute_statement ? stmt->execute_statement : stmt->statement;
 	q->statement_type = stmt->statement_type;
 	q->opos = 0;
 	q->from_pos = -1;
@@ -1274,6 +1362,7 @@ QP_initialize(QueryParse *q, const StatementClass *stmt)
 #define	FLGB_INACCURATE_RESULT	(1L << 1)
 #define	FLGB_CREATE_KEYSET	(1L << 2)
 #define	FLGB_KEYSET_DRIVEN	(1L << 3)
+#define	FLGB_BUILDING_PREPARE_STATEMENT	(1L << 4)
 typedef struct _QueryBuild {
 	char	*query_statement;
 	UInt4	str_size_limit;
@@ -1282,6 +1371,7 @@ typedef struct _QueryBuild {
 	int	current_row;
 	int	param_number;
 	APDFields *apdopts;
+	IPDFields *ipdopts;
 	UInt4	load_stmt_len;
 	UInt4	flags;
 	BOOL	lf_conv;
@@ -1303,11 +1393,13 @@ QB_initialize(QueryBuild *qb, UInt4 size, StatementClass *stmt, ConnectionClass
 	qb->load_stmt_len = 0;
 	qb->stmt = stmt;
 	qb->apdopts = NULL;
+	qb->ipdopts = NULL;
 	if (conn)
 		qb->conn = conn;
 	else if (stmt)
 	{
 		qb->apdopts = SC_get_APD(stmt);
+		qb->ipdopts = SC_get_IPD(stmt);
 		qb->conn = SC_get_conn(stmt);
 		if (stmt->pre_executing)
 			qb->flags |= FLGB_PRE_EXECUTING;
@@ -1658,6 +1750,106 @@ insert_without_target(const char *stmt, int *endpos)
 		|| ';' == wstmt[0];
 }
 
+static	int
+Prepare_and_convert(StatementClass *stmt, QueryParse *qp, QueryBuild *qb)
+{
+	const static char *func = "Prepare_and_convert";
+	char	*new_statement, *exe_statement = NULL;
+	int	retval;
+
+	if (QB_initialize(qb, qp->stmt_len, stmt, NULL) < 0)
+		return SQL_ERROR;
+	if (!stmt->prepared) /*  not yet prepared */
+	{
+		int	i, elen;
+		SWORD	marker_count;
+		const IPDFields *ipdopts = qb->ipdopts;
+
+		new_statement = qb->query_statement;
+		qb->flags = FLGB_BUILDING_PREPARE_STATEMENT;
+		sprintf(new_statement, "PREPARE _PLAN%0x", stmt);
+		qb->npos = strlen(new_statement);
+		if (SQL_SUCCESS != PGAPI_NumParams(stmt, &marker_count))
+		{
+			QB_Destructor(qb);
+			return SQL_ERROR;
+		}
+		if (marker_count > 0)
+		{
+			CVT_APPEND_CHAR(qb, '(');
+			for (i = 0; i < marker_count; i++)
+			{
+				if (i > 0)
+					CVT_APPEND_STR(qb, ", ");
+				CVT_APPEND_STR(qb, pgtype_to_name(stmt, ipdopts->parameters[i].PGType));
+			}
+			CVT_APPEND_CHAR(qb, ')');
+		}
+		CVT_APPEND_STR(qb, " as ");
+		for (qp->opos = 0; qp->opos < qp->stmt_len; qp->opos++)
+		{
+			retval = inner_process_tokens(qp, qb);
+			if (SQL_ERROR == retval)
+			{
+				if (0 == SC_get_errornumber(stmt))
+				{
+					SC_set_error(stmt, qb->errornumber, qb->errormsg);
+				}
+				SC_log_error(func, "", stmt);
+				QB_Destructor(qb);
+				return retval;
+			}
+		}
+		CVT_APPEND_CHAR(qb, ';');
+		/* build the execute statement */
+		exe_statement = malloc(30 + 2 * marker_count);
+		sprintf(exe_statement, "EXECUTE _PLAN%0x", stmt);
+		if (marker_count > 0)
+		{
+			elen = strlen(exe_statement);
+			exe_statement[elen++] = '(';
+			for (i = 0; i < marker_count; i++)
+			{
+				if (i > 0)
+					exe_statement[elen++] = ',';
+				exe_statement[elen++]  = '?';
+			}
+			exe_statement[elen++] = ')';
+			exe_statement[elen] = '\0';
+		}
+		stmt->execute_statement = exe_statement;
+		QP_initialize(qp, stmt);
+	}
+	qb->flags = 0;
+	qb->param_number = -1;
+	for (qp->opos = 0; qp->opos < qp->stmt_len; qp->opos++)
+	{
+		retval = inner_process_tokens(qp, qb);
+		if (SQL_ERROR == retval)
+		{
+			if (0 == SC_get_errornumber(stmt))
+			{
+				SC_set_error(stmt, qb->errornumber, qb->errormsg);
+			}
+			SC_log_error(func, "", stmt);
+			if (exe_statement)
+			{
+				free(exe_statement);
+				stmt->execute_statement = NULL;
+			}
+			QB_Destructor(qb);
+			return retval;
+		}
+	}
+	/* make sure new_statement is always null-terminated */
+	CVT_TERMINATE(qb);
+
+	if (exe_statement)
+		SC_set_prepare_before_exec(stmt);
+	stmt->stmt_with_params = qb->query_statement;
+	return SQL_SUCCESS;
+}
+
 #define		my_strchr(conn, s1,c1) pg_mbschr(conn->ccsc, s1,c1)
 
 /*
@@ -1666,7 +1858,7 @@ insert_without_target(const char *stmt, int *endpos)
  *	This function does a dynamic memory allocation to get rid of query size limit!
  */
 int
-copy_statement_with_parameters(StatementClass *stmt)
+copy_statement_with_parameters(StatementClass *stmt, BOOL buildPrepareStatement)
 {
 	static char *func = "copy_statement_with_parameters";
 	RETCODE		retval;
@@ -1690,12 +1882,6 @@ copy_statement_with_parameters(StatementClass *stmt)
 	qp = &query_org;
 	QP_initialize(qp, stmt);
 
-	if (ci->disallow_premature)
-		prepare_dummy_cursor = stmt->pre_executing;
-	if (prepare_dummy_cursor)
-		qp->flags |= FLGP_PREPARE_DUMMY_CURSOR;
-
-
 #ifdef	DRIVER_CURSOR_IMPLEMENT
 	if (stmt->statement_type != STMT_TYPE_SELECT)
 	{
@@ -1731,6 +1917,7 @@ copy_statement_with_parameters(StatementClass *stmt)
 		stmt->options.cursor_type = SQL_CURSOR_STATIC;
 #endif   /* DRIVER_CURSOR_IMPLEMENT */
 
+	stmt->miscinfo = 0;
 	/* If the application hasn't set a cursor name, then generate one */
 	if (stmt->cursor_name[0] == '\0')
 		sprintf(stmt->cursor_name, "SQL_CUR%p", stmt);
@@ -1739,12 +1926,21 @@ copy_statement_with_parameters(StatementClass *stmt)
 		free(stmt->stmt_with_params);
 		stmt->stmt_with_params = NULL;
 	}
+
 	qb = &query_crt;
+	if (stmt->prepared || (buildPrepareStatement && stmt->options.scroll_concurrency == SQL_CONCUR_READ_ONLY))
+	{
+		return Prepare_and_convert(stmt, qp, qb);
+	}
+
+	if (ci->disallow_premature)
+		prepare_dummy_cursor = stmt->pre_executing;
+	if (prepare_dummy_cursor)
+		qp->flags |= FLGP_PREPARE_DUMMY_CURSOR;
 	if (QB_initialize(qb, qp->stmt_len, stmt, NULL) < 0)
 		return SQL_ERROR;
 	new_statement = qb->query_statement;
 
-	stmt->miscinfo = 0;
 	/* For selects, prepend a declare cursor to the statement */
 	if (stmt->statement_type == STMT_TYPE_SELECT)
 	{
@@ -2183,11 +2379,13 @@ ResolveOneParam(QueryBuild *qb)
 
 	ConnectionClass *conn = qb->conn;
 	ConnInfo   *ci = &(conn->connInfo);
-	APDFields *opts = qb->apdopts;
+	const APDFields *apdopts = qb->apdopts;
+	const IPDFields *ipdopts = qb->ipdopts;
 
 	int		param_number;
 	char		param_string[128], tmp[256],
 			cbuf[PG_NUMERIC_MAX_PRECISION * 2]; /* seems big enough to handle the data in this function */
+	Int4		param_pgtype;
 	Int2		param_ctype, param_sqltype;
 	SIMPLE_TIME	st;
 	time_t		t;
@@ -2199,15 +2397,16 @@ ResolveOneParam(QueryBuild *qb)
 	char		*buffer, *buf, *allocbuf;
 	Oid		lobj_oid;
 	int		lobj_fd, retval;
-	UInt4	offset = opts->param_offset_ptr ? *opts->param_offset_ptr : 0;
+	UInt4	offset = apdopts->param_offset_ptr ? *apdopts->param_offset_ptr : 0;
 	UInt4	current_row = qb->current_row;
+	BOOL	handling_large_object = FALSE;
 
 	/*
 	 * Its a '?' parameter alright
 	 */
 	param_number = ++qb->param_number;
 
-	if (param_number >= opts->allocated)
+	if (param_number >= apdopts->allocated)
 	{
 		if (0 != (qb->flags & FLGB_PRE_EXECUTING))
 		{
@@ -2217,40 +2416,59 @@ ResolveOneParam(QueryBuild *qb)
 		}
 		else
 		{
-			CVT_APPEND_CHAR(qb, '?');
-			return SQL_SUCCESS;
+			qb->errormsg = "The # of binded parameters < the # of parameter markers";
+			qb->errornumber = STMT_COUNT_FIELD_INCORRECT;
+			CVT_TERMINATE(qb);	/* just in case */
+			return SQL_ERROR;
 		}
 	}
-
-	/* Assign correct buffers based on data at exec param or not */
-	if (opts->parameters[param_number].data_at_exec)
+	if (SQL_PARAM_OUTPUT == apdopts->parameters[param_number].paramType)
 	{
-		used = opts->parameters[param_number].EXEC_used ? *opts->parameters[param_number].EXEC_used : SQL_NTS;
-		buffer = opts->parameters[param_number].EXEC_buffer;
+		qb->errormsg = "Output parameter isn't available";
+		qb->errornumber = STMT_NOT_IMPLEMENTED_ERROR;
+		CVT_TERMINATE(qb);	/* just in case */
+		return SQL_ERROR;
+	}
+
+	if (0 != (qb->flags & FLGB_BUILDING_PREPARE_STATEMENT))
+	{
+		char	pnum[16];
+
+		sprintf(pnum, "$%d", param_number + 1);
+		CVT_APPEND_STR(qb, pnum); 
+		return SQL_SUCCESS;
+	} 
+	/* Assign correct buffers based on data at exec param or not */
+	if (apdopts->parameters[param_number].data_at_exec)
+	{
+		used = apdopts->parameters[param_number].EXEC_used ? *apdopts->parameters[param_number].EXEC_used : SQL_NTS;
+		buffer = apdopts->parameters[param_number].EXEC_buffer;
+		if (apdopts->parameters[param_number].lobj_oid)
+			handling_large_object = TRUE;
 	}
 	else
 	{
-		UInt4	bind_size = opts->param_bind_type;
+		UInt4	bind_size = apdopts->param_bind_type;
 		UInt4	ctypelen;
 
-		buffer = opts->parameters[param_number].buffer + offset;
+		buffer = apdopts->parameters[param_number].buffer + offset;
 		if (current_row > 0)
 		{
 			if (bind_size > 0)
 				buffer += (bind_size * current_row);
-			else if (ctypelen = ctype_length(opts->parameters[param_number].CType), ctypelen > 0)
+			else if (ctypelen = ctype_length(apdopts->parameters[param_number].CType), ctypelen > 0)
 				buffer += current_row * ctypelen;
 			else 
-				buffer += current_row * opts->parameters[param_number].buflen;
+				buffer += current_row * apdopts->parameters[param_number].buflen;
 		}
-		if (opts->parameters[param_number].used)
+		if (apdopts->parameters[param_number].used)
 		{
 			UInt4	p_offset = offset;
 			if (bind_size > 0)
 				p_offset = offset + bind_size * current_row;
 			else
 				p_offset = offset + sizeof(SDWORD) * current_row;
-			used = *(SDWORD *)((char *)opts->parameters[param_number].used + p_offset);
+			used = *(SDWORD *)((char *)apdopts->parameters[param_number].used + p_offset);
 		}
 		else
 			used = SQL_NTS;
@@ -2262,6 +2480,11 @@ ResolveOneParam(QueryBuild *qb)
 		CVT_APPEND_STR(qb, "NULL");
 		return SQL_SUCCESS;
 	}
+	/* Handle DEFAULT_PARAM parameter data */
+	if (used == SQL_DEFAULT_PARAM)
+	{
+		return SQL_SUCCESS;
+	}
 
 	/*
 	 * If no buffer, and it's not null, then what the hell is it? Just
@@ -2275,28 +2498,29 @@ ResolveOneParam(QueryBuild *qb)
 			qb->flags |= FLGB_INACCURATE_RESULT;
 			return SQL_SUCCESS;
 		}
-		else
+		else if (!handling_large_object)
 		{
 			CVT_APPEND_CHAR(qb, '?');
 			return SQL_SUCCESS;
 		}
 	}
 
-	param_ctype = opts->parameters[param_number].CType;
-	param_sqltype = opts->parameters[param_number].SQLType;
+	param_ctype = apdopts->parameters[param_number].CType;
+	param_sqltype = ipdopts->parameters[param_number].SQLType;
+	param_pgtype = ipdopts->parameters[param_number].PGType;
 
 	mylog("%s: from(fcType)=%d, to(fSqlType)=%d\n", func,
 				param_ctype, param_sqltype);
 
 	/* replace DEFAULT with something we can use */
 	if (param_ctype == SQL_C_DEFAULT)
-		param_ctype = sqltype_to_default_ctype(param_sqltype);
+		param_ctype = sqltype_to_default_ctype(conn, param_sqltype);
 
 	allocbuf = buf = NULL;
 	param_string[0] = '\0';
 	cbuf[0] = '\0';
 	memset(&st, 0, sizeof(st));
-	t = time(NULL);
+	t = SC_get_time(qb->stmt);
 #ifdef	HAVE_LOCALTIME_R
 	tim = localtime_r(&t, &tm);
 #else
@@ -2310,9 +2534,31 @@ ResolveOneParam(QueryBuild *qb)
 	switch (param_ctype)
 	{
 		case SQL_C_BINARY:
-		case SQL_C_CHAR:
 			buf = buffer;
 			break;
+		case SQL_C_CHAR:
+#ifdef	WIN_UNICODE_SUPPORT
+			switch (param_sqltype)
+			{
+				case SQL_WCHAR:
+				case SQL_WVARCHAR:
+				case SQL_WLONGVARCHAR:
+					if (SQL_NTS == used)
+						used = strlen(buffer);
+					allocbuf = malloc(2 * (used + 1));
+					used = MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, buffer,
+						used, (LPWSTR) allocbuf, used + 1);
+					buf = ucs2_to_utf8((SQLWCHAR *) allocbuf, used, &used);
+					free(allocbuf);
+					allocbuf = buf;
+					break;
+				default:
+					buf = buffer;
+			}
+#else
+			buf = buffer;
+#endif /* WIN_UNICODE_SUPPORT */
+			break;
 
 #ifdef	UNICODE_SUPPORT
 		case SQL_C_WCHAR:
@@ -2338,18 +2584,16 @@ ResolveOneParam(QueryBuild *qb)
 			break;
 
 #if (ODBCVER >= 0x0300) && defined(ODBCINT64)
-#ifdef WIN32
 		case SQL_C_SBIGINT:
-			sprintf(param_string, "%I64d",
+			sprintf(param_string, FORMATI64,
 					*((SQLBIGINT *) buffer));
 			break;
 
 		case SQL_C_UBIGINT:
-			sprintf(param_string, "%I64u",
+			sprintf(param_string, FORMATI64U,
 					*((SQLUBIGINT *) buffer));
 			break;
 
-#endif /* WIN32 */
 #endif /* ODBCINT64 */
 		case SQL_C_SSHORT:
 		case SQL_C_SHORT:
@@ -2541,22 +2785,56 @@ ResolveOneParam(QueryBuild *qb)
 			break;
 
 		case SQL_BINARY:
-		case SQL_VARBINARY:/* non-ascii characters should be
-							* converted to octal */
-			CVT_APPEND_CHAR(qb, '\'');	/* Open Quote */
-
-			mylog("SQL_VARBINARY: about to call convert_to_pgbinary, used = %d\n", used);
-
-			CVT_APPEND_BINARY(qb, buf, used);
-
-			CVT_APPEND_CHAR(qb, '\'');	/* Close Quote */
-
-			break;
-
+		case SQL_VARBINARY:
 		case SQL_LONGVARBINARY:
+			switch (param_ctype)
+			{
+				case SQL_C_BINARY:
+					break;
+				case SQL_C_CHAR:
+					switch (used)
+					{
+						case SQL_NTS:
+							used = strlen(buf);
+							break;
+					}
+					allocbuf = malloc(used / 2 + 1);
+					if (allocbuf)
+					{
+						pg_hex2bin(buf, allocbuf, used);
+						buf = allocbuf;
+						used /= 2;
+					}
+					break;
+				default:
+					qb->errormsg = "Could not convert the ctype to binary type";
+					qb->errornumber = STMT_EXEC_ERROR;
+					return SQL_ERROR;
+			}
+			if (param_pgtype == PG_TYPE_BYTEA)
+			{
+				/* non-ascii characters should be
+				 * converted to octal
+				 */
+				CVT_APPEND_CHAR(qb, '\'');	/* Open Quote */
 
-			if (opts->parameters[param_number].data_at_exec)
-				lobj_oid = opts->parameters[param_number].lobj_oid;
+				mylog("SQL_VARBINARY: about to call convert_to_pgbinary, used = %d\n", used);
+
+				CVT_APPEND_BINARY(qb, buf, used);
+
+				CVT_APPEND_CHAR(qb, '\'');	/* Close Quote */
+
+				break;
+			}
+			if (param_pgtype != conn->lobj_type)
+			{
+				qb->errormsg = "Could not convert binary other than LO type";
+				qb->errornumber = STMT_EXEC_ERROR;
+				return SQL_ERROR;
+			}
+
+			if (apdopts->parameters[param_number].data_at_exec)
+				lobj_oid = apdopts->parameters[param_number].lobj_oid;
 			else
 			{
 				/* begin transaction if needed */
@@ -2609,7 +2887,7 @@ ResolveOneParam(QueryBuild *qb)
 			 * parameter marker -- the data has already been sent to
 			 * the large object
 			 */
-			sprintf(param_string, "'%d'", lobj_oid);
+			sprintf(param_string, "'%d'::lo", lobj_oid);
 			CVT_APPEND_STR(qb, param_string);
 
 			break;
@@ -2671,10 +2949,8 @@ ResolveOneParam(QueryBuild *qb)
 
 			break;
 	}
-#ifdef	UNICODE_SUPPORT
 	if (allocbuf)
 		free(allocbuf);
-#endif /* UNICODE_SUPPORT */
 	return SQL_SUCCESS;
 }
 
@@ -3275,22 +3551,32 @@ convert_from_pgbinary(const unsigned char *value, unsigned char *rgbValue, int c
 		{
 			if (value[i + 1] == '\\')
 			{
-				rgbValue[o] = value[i];
+				if (rgbValue)
+					rgbValue[o] = value[i];
 				i += 2;
 			}
 			else
 			{
-				rgbValue[o] = conv_from_octal(&value[i]);
+				if (rgbValue)
+					rgbValue[o] = conv_from_octal(&value[i]);
 				i += 4;
 			}
 		}
 		else
-			rgbValue[o] = value[i++];
-		mylog("convert_from_pgbinary: i=%d, rgbValue[%d] = %d, %c\n", i, o, rgbValue[o], rgbValue[o]);
+		{
+			if (rgbValue)
+				rgbValue[o] = value[i];
+			i++;
+		}
+		/** if (rgbValue)
+			mylog("convert_from_pgbinary: i=%d, rgbValue[%d] = %d, %c\n", i, o, rgbValue[o], rgbValue[o]); ***/
 		o++;
 	}
 
-	rgbValue[o] = '\0';			/* extra protection */
+	if (rgbValue)
+		rgbValue[o] = '\0';		/* extra protection */
+
+	mylog("convert_from_pgbinary: in=%d, out = %d\n", ilen, o);
 
 	return o;
 }
@@ -3430,6 +3716,37 @@ pg_bin2hex(UCHAR *src, UCHAR *dst, int length)
 	return length;
 }
 
+int
+pg_hex2bin(const UCHAR *src, UCHAR *dst, int length)
+{
+	UCHAR		chr;
+	const UCHAR	*src_wk;
+	UCHAR		*dst_wk;
+	int		i, val;
+	BOOL		HByte = TRUE;
+
+	for (i = 0, src_wk = src, dst_wk = dst; i < length; i++, src_wk++)
+	{
+		chr = *src_wk;
+		if (chr >= 'a' && chr <= 'f')
+			val = chr - 'a' + 10;
+		else if (chr >= 'A' && chr <= 'F')
+			val = chr - 'A' + 10;
+		else
+			val = chr - '0';
+		if (HByte)
+			*dst_wk = (val << 4);
+		else
+		{
+			*dst_wk += val; 
+			dst_wk++;
+		}
+		HByte = !HByte;
+	}
+	*dst_wk = '\0';
+	return length;
+}
+
 /*-------
  *	1. get oid (from 'value')
  *	2. open the large object
@@ -3458,8 +3775,20 @@ convert_lo(StatementClass *stmt, const void *value, Int2 fCType, PTR rgbValue,
 	ConnectionClass *conn = SC_get_conn(stmt);
 	ConnInfo   *ci = &(conn->connInfo);
 	ARDFields	*opts = SC_get_ARD(stmt);
-	int			factor = (fCType == SQL_C_CHAR ? 2 : 1);
+	int			factor;
 
+	switch (fCType)
+	{
+		case SQL_C_CHAR:
+			factor = 2;
+			break;
+		case SQL_C_BINARY:
+			factor = 1;
+			break;
+		default:
+			SC_set_error(stmt, STMT_EXEC_ERROR, "Could not convert lo to the c-type");
+			return COPY_GENERAL_ERROR;
+	}
 	/* If using SQLGetData, then current_col will be set */
 	if (stmt->current_col >= 0)
 	{
