@@ -48,7 +48,7 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 	/* fundamental info */
 	sprintf(connect_string, "%s=%s;DATABASE=%s;SERVER=%s;PORT=%s;UID=%s;PWD=%s",
 			got_dsn ? "DSN" : "DRIVER",
-			got_dsn ? ci->dsn : ci->driver,
+			got_dsn ? ci->dsn : ci->drivername,
 			ci->database,
 			ci->server,
 			ci->port,
@@ -61,7 +61,7 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 	hlen = strlen(connect_string);
 	if (!abbrev)
 		sprintf(&connect_string[hlen],
-				";%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%s;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d",
+				";%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%s;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%s;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d;%s=%d",
 				INI_READONLY,
 				ci->onlyread,
 				INI_PROTOCOL,
@@ -117,7 +117,11 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 				INI_TRUEISMINUS1,
 				ci->true_is_minus1,
 				INI_INT8AS,
-				ci->int8_as);
+				ci->int8_as,
+				INI_BYTEAASLONGVARBINARY,
+				ci->bytea_as_longvarbinary,
+				INI_USESERVERSIDEPREPARE,
+				ci->use_server_side_prepare);
 	/* Abbrebiation is needed ? */
 	if (abbrev || strlen(connect_string) >= len)
 	{
@@ -175,6 +179,10 @@ makeConnectString(char *connect_string, const ConnInfo *ci, UWORD len)
 			flag |= BIT_FAKEOIDINDEX;
 		if (ci->true_is_minus1)
 			flag |= BIT_TRUEISMINUS1;
+		if (ci->bytea_as_longvarbinary)
+			flag |= BIT_BYTEAASLONGVARBINARY;
+		if (ci->use_server_side_prepare)
+			flag |= BIT_USESERVERSIDEPREPARE;
 
 		sprintf(&connect_string[hlen],
 				";A6=%s;A7=%d;A8=%d;B0=%d;B1=%d;%s=%d;C2=%s;CX=%02x%lx",
@@ -244,6 +252,8 @@ unfoldCXAttribute(ConnInfo *ci, const char *value)
 	sprintf(ci->show_oid_column, "%d", (char)((flag & BIT_SHOWOIDCOLUMN) != 0));
 	sprintf(ci->fake_oid_index, "%d", (char)((flag & BIT_FAKEOIDINDEX) != 0));
 	ci->true_is_minus1 = (char)((flag & BIT_TRUEISMINUS1) != 0);
+	ci->bytea_as_longvarbinary = (char)((flag & BIT_BYTEAASLONGVARBINARY) != 0);
+	ci->use_server_side_prepare = (char)((flag & BIT_USESERVERSIDEPREPARE) != 0);
 }
 void
 copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
@@ -252,7 +262,7 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		strcpy(ci->dsn, value);
 
 	else if (stricmp(attribute, "driver") == 0)
-		strcpy(ci->driver, value);
+		strcpy(ci->drivername, value);
 
 	else if (stricmp(attribute, INI_DATABASE) == 0)
 		strcpy(ci->database, value);
@@ -302,10 +312,14 @@ copyAttributes(ConnInfo *ci, const char *attribute, const char *value)
 		ci->true_is_minus1 = atoi(value);
 	else if (stricmp(attribute, INI_INT8AS) == 0)
 		ci->int8_as = atoi(value);
+	else if (stricmp(attribute, INI_BYTEAASLONGVARBINARY) == 0)
+		ci->bytea_as_longvarbinary = atoi(value);
+	else if (stricmp(attribute, INI_USESERVERSIDEPREPARE) == 0)
+		ci->use_server_side_prepare = atoi(value);
 	else if (stricmp(attribute, "CX") == 0)
 		unfoldCXAttribute(ci, value);
 
-	mylog("copyAttributes: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", ci->dsn, ci->server, ci->database, ci->username, ci->password, ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
+	mylog("copyAttributes: DSN='%s',server='%s',dbase='%s',user='%s',passwd='%s',port='%s',onlyread='%s',protocol='%s',conn_settings='%s',disallow_premature=%d)\n", ci->dsn, ci->server, ci->database, ci->username, ci->password ? "xxxxx" : "", ci->port, ci->onlyread, ci->protocol, ci->conn_settings, ci->disallow_premature);
 }
 
 void
@@ -405,8 +419,17 @@ getDSNdefaults(ConnInfo *ci)
 		ci->true_is_minus1 = DEFAULT_TRUEISMINUS1;
 	if (ci->int8_as < -100)
 		ci->int8_as = DEFAULT_INT8AS;
+	if (ci->bytea_as_longvarbinary < 0)
+		ci->bytea_as_longvarbinary = DEFAULT_BYTEAASLONGVARBINARY;
+	if (ci->use_server_side_prepare < 0)
+		ci->use_server_side_prepare = DEFAULT_USESERVERSIDEPREPARE;
 }
 
+int
+getDriverNameFromDSN(const char *dsn, char *driver_name, int namelen)
+{
+	return SQLGetPrivateProfileString(ODBC_DATASOURCES, dsn, "", driver_name, namelen, ODBC_INI);
+}
 
 void
 getDSNinfo(ConnInfo *ci, char overwrite)
@@ -421,7 +444,7 @@ getDSNinfo(ConnInfo *ci, char overwrite)
  */
 	if (DSN[0] == '\0')
 	{
-		if (ci->driver[0] != '\0')
+		if (ci->drivername[0] != '\0')
 			return;
 		else
 			strcpy(DSN, INI_DSN);
@@ -431,6 +454,13 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 	while (*(DSN + strlen(DSN) - 1) == ' ')
 		*(DSN + strlen(DSN) - 1) = '\0';
 
+	if (ci->drivername[0] == '\0' || overwrite)
+	{
+		getDriverNameFromDSN(DSN, ci->drivername, sizeof(ci->drivername));
+		if (ci->drivername[0] && stricmp(ci->drivername, DBMS_NAME))
+			getCommonDefaults(ci->drivername, ODBCINST_INI, ci);
+	}
+
 	/* Proceed with getting info for the given DSN. */
 
 	if (ci->desc[0] == '\0' || overwrite)
@@ -516,6 +546,20 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 			ci->int8_as = atoi(temp);
 	}
 
+	if (ci->bytea_as_longvarbinary < 0 || overwrite)
+	{
+		SQLGetPrivateProfileString(DSN, INI_BYTEAASLONGVARBINARY, "", temp, sizeof(temp), ODBC_INI);
+		if (temp[0])
+			ci->bytea_as_longvarbinary = atoi(temp);
+	}
+
+	if (ci->use_server_side_prepare < 0 || overwrite)
+	{
+		SQLGetPrivateProfileString(DSN, INI_USESERVERSIDEPREPARE, "", temp, sizeof(temp), ODBC_INI);
+		if (temp[0])
+			ci->use_server_side_prepare = atoi(temp);
+	}
+
 	/* Allow override of odbcinst.ini parameters here */
 	getCommonDefaults(DSN, ODBC_INI, ci);
 
@@ -525,7 +569,7 @@ getDSNinfo(ConnInfo *ci, char overwrite)
 		 ci->port,
 		 ci->database,
 		 ci->username,
-		 ci->password);
+		 ci->password ? "xxxxx" : "");
 	qlog("          onlyread='%s',protocol='%s',showoid='%s',fakeoidindex='%s',showsystable='%s'\n",
 		 ci->onlyread,
 		 ci->protocol,
@@ -547,33 +591,14 @@ getDSNinfo(ConnInfo *ci, char overwrite)
  *	to the ODBCINST.INI portion of the registry
  */
 void
-writeDriverCommoninfo(const ConnInfo *ci)
+writeDriverCommoninfo(const char *fileName, const char *sectionName,
+			 const GLOBAL_VALUES *comval)
 {
-	const char *sectionName;
-	const char *fileName;
-	const GLOBAL_VALUES *comval;
 	char		tmp[128];
 
-	if (ci)
-		if (ci->dsn && ci->dsn[0])
-		{
-			mylog("DSN=%s updating\n", ci->dsn);
-			comval = &(ci->drivers);
-			sectionName = ci->dsn;
-			fileName = ODBC_INI;
-		}
-		else
-		{
-			mylog("ci but dsn==NULL\n");
-			return;
-		}
-	else
-	{
-		mylog("drivers updating\n");
-		comval = &globals;
+	if (ODBCINST_INI == fileName && NULL == sectionName)
 		sectionName = DBMS_NAME;
-		fileName = ODBCINST_INI;
-	}
+ 
 	sprintf(tmp, "%d", comval->fetch_max);
 	SQLWritePrivateProfileString(sectionName,
 								 INI_FETCH, tmp, fileName);
@@ -599,7 +624,7 @@ writeDriverCommoninfo(const ConnInfo *ci)
 	/*
 	 * Never update the onlyread from this module.
 	 */
-	if (!ci)
+	if (ODBCINST_INI == fileName)
 	{
 		sprintf(tmp, "%d", comval->onlyread);
 		SQLWritePrivateProfileString(sectionName, INI_READONLY, tmp,
@@ -752,6 +777,16 @@ writeDSNinfo(const ConnInfo *ci)
 								 INI_INT8AS,
 								 temp,
 								 ODBC_INI);
+	sprintf(temp, "%d", ci->bytea_as_longvarbinary);
+	SQLWritePrivateProfileString(DSN,
+								 INI_BYTEAASLONGVARBINARY,
+								 temp,
+								 ODBC_INI);
+	sprintf(temp, "%d", ci->use_server_side_prepare);
+	SQLWritePrivateProfileString(DSN,
+								 INI_USESERVERSIDEPREPARE,
+								 temp,
+								 ODBC_INI);
 }
 
 
