@@ -31,6 +31,7 @@
 #define ODBCINST_INI					"ODBCINST.INI"
 #endif
 
+#define	ODBC_DATASOURCES	"ODBC Data Sources"
 
 #if (ODBCVER >= 0x0300)
 #ifdef  UNICODE_SUPPORT
@@ -99,6 +100,8 @@
 #define INI_LFCONVERSION			"LFConversion"
 #define INI_TRUEISMINUS1			"TrueIsMinus1"
 #define INI_INT8AS				"BI"
+#define INI_BYTEAASLONGVARBINARY		"ByteaAsLongVarBinary"
+#define INI_USESERVERSIDEPREPARE		"UseServerSidePrepare"
 /* Bit representaion for abbreviated connection strings */
 #define BIT_LFCONVERSION			(1L)
 #define BIT_UPDATABLECURSORS			(1L<<1)
@@ -124,8 +127,10 @@
 #define BIT_SHOWOIDCOLUMN			(1L<<21)
 #define BIT_FAKEOIDINDEX			(1L<<22)
 #define BIT_TRUEISMINUS1			(1L<<23)
+#define BIT_BYTEAASLONGVARBINARY		(1L<<24)
+#define BIT_USESERVERSIDEPREPARE		(1L<<25)
 
-#define EFFECTIVE_BIT_COUNT			24
+#define EFFECTIVE_BIT_COUNT			26
 
 
 /*	Connection Defaults */
@@ -169,6 +174,8 @@
 #define DEFAULT_LFCONVERSION		0
 #endif	/* WIN32 */
 #define DEFAULT_INT8AS			0
+#define DEFAULT_BYTEAASLONGVARBINARY	0
+#define DEFAULT_USESERVERSIDEPREPARE	0
 
 /*	prototypes */
 void		getCommonDefaults(const char *section, const char *filename, ConnInfo *ci);
@@ -193,15 +200,22 @@ int CALLBACK ds_options2Proc(HWND hdlg,
 			   UINT wMsg,
 			   WPARAM wParam,
 			   LPARAM lParam);
+int CALLBACK manage_dsnProc(HWND hdlg,
+			   UINT wMsg,
+			   WPARAM wParam,
+			   LPARAM lParam);
 #endif   /* WIN32 */
 
 void		updateGlobals(void);
-void		writeDriverCommoninfo(const ConnInfo *ci);
+void		writeDriverCommoninfo(const char *fileName, const char *sectionName,
+		const GLOBAL_VALUES *);
 void		writeDSNinfo(const ConnInfo *ci);
 void		getDSNdefaults(ConnInfo *ci);
 void		getDSNinfo(ConnInfo *ci, char overwrite);
 void		makeConnectString(char *connect_string, const ConnInfo *ci, UWORD);
 void		copyAttributes(ConnInfo *ci, const char *attribute, const char *value);
 void		copyCommonAttributes(ConnInfo *ci, const char *attribute, const char *value);
+int	getDriverNameFromDSN(const char *dsn, char *driver_name, int namelen);
+int     changeDriverNameOfaDSN(const char *dsn, const char *driver_name, DWORD *errcode);
 
 #endif
