@@ -66,7 +66,7 @@ Int4 pgtypes_defined[]	= {
 				PG_TYPE_MONEY,
 				PG_TYPE_BOOL,
 				PG_TYPE_BYTEA,
-				PG_TYPE_LO,
+				PG_TYPE_LO_UNDEFINED,
 				0 };
 */
 
@@ -79,9 +79,8 @@ Int2		sqlTypes[] = {
 	SQL_CHAR,
 #if (ODBCVER >= 0x0300)
 	SQL_TYPE_DATE,
-#else
-	SQL_DATE,
 #endif /* ODBCVER */
+	SQL_DATE,
 	SQL_DECIMAL,
 	SQL_DOUBLE,
 	SQL_FLOAT,
@@ -94,10 +93,9 @@ Int2		sqlTypes[] = {
 #if (ODBCVER >= 0x0300)
 	SQL_TYPE_TIME,
 	SQL_TYPE_TIMESTAMP,
-#else
+#endif /* ODBCVER */
 	SQL_TIME,
 	SQL_TIMESTAMP,
-#endif /* ODBCVER */
 	SQL_TINYINT,
 	SQL_VARBINARY,
 	SQL_VARCHAR,
@@ -109,9 +107,9 @@ Int2		sqlTypes[] = {
 	0
 };
 
-#if (ODBCVER >= 0x0300) && defined(OBDCINT64)
-/* #define	ALLOWED_C_BIGINT	SQL_C_SBIGINT */
-#define	ALLOWED_C_BIGINT	SQL_C_CHAR /* Delphi should be either ? */
+#if (ODBCVER >= 0x0300) && defined(ODBCINT64)
+#define	ALLOWED_C_BIGINT	SQL_C_SBIGINT
+/* #define	ALLOWED_C_BIGINT	SQL_C_CHAR */ /* Delphi should be either ? */
 #else
 #define	ALLOWED_C_BIGINT	SQL_C_CHAR
 #endif
@@ -143,9 +141,10 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			pgType = ci->drivers.bools_as_char ? PG_TYPE_CHAR : PG_TYPE_BOOL;
 			break;
 
-		case SQL_DATE:
 #if (ODBCVER >= 0x0300)
 		case SQL_TYPE_DATE:
+#else
+		case SQL_DATE:
 #endif /* ODBCVER */
 			pgType = PG_TYPE_DATE;
 			break;
@@ -169,7 +168,10 @@ sqltype_to_pgtype(StatementClass *stmt, SWORD fSqlType)
 			break;
 
 		case SQL_LONGVARBINARY:
-			pgType = PG_TYPE_LO;
+			if (ci->bytea_as_longvarbinary)
+				pgType = PG_TYPE_BYTEA;
+			else
+				pgType = conn->lobj_type;
 			break;
 
 		case SQL_LONGVARCHAR:
@@ -282,8 +284,11 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 #endif /* UNICODE_SUPPORT */
 
 		case PG_TYPE_BYTEA:
-			return SQL_VARBINARY;
-		case PG_TYPE_LO:
+			if (ci->bytea_as_longvarbinary)
+				return SQL_LONGVARBINARY;
+			else
+				return SQL_VARBINARY;
+		case PG_TYPE_LO_UNDEFINED:
 			return SQL_LONGVARBINARY;
 
 		case PG_TYPE_INT2:
@@ -298,8 +303,8 @@ pgtype_to_concise_type(StatementClass *stmt, Int4 type)
 		case PG_TYPE_INT8:
 			if (ci->int8_as != 0) 
 				return ci->int8_as;
-			if (conn->ms_jet) 
-				return SQL_NUMERIC; /* maybe a little better than SQL_VARCHAR */
+			/***if (conn->ms_jet) 
+				return SQL_NUMERIC; ***/ /* maybe a little better than SQL_VARCHAR */
 #if (ODBCVER >= 0x0300)
 			return SQL_BIGINT;
 #else
@@ -446,17 +451,15 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 
 		case PG_TYPE_BYTEA:
 			return SQL_C_BINARY;
-		case PG_TYPE_LO:
+		case PG_TYPE_LO_UNDEFINED:
 			return SQL_C_BINARY;
 #ifdef	UNICODE_SUPPORT
 		case PG_TYPE_BPCHAR:
 		case PG_TYPE_VARCHAR:
 		case PG_TYPE_TEXT:
-			return conn->unicode ? SQL_C_WCHAR : SQL_C_CHAR;
-			/*	The following seems worse.
-			 *
+			if (conn->unicode && ! conn->ms_jet && ! stmt->manual_result)
+				return SQL_C_WCHAR;
 			return SQL_C_CHAR;
-			 */
 #endif /* UNICODE_SUPPORT */
 
 		default:
@@ -469,7 +472,7 @@ pgtype_to_ctype(StatementClass *stmt, Int4 type)
 }
 
 
-char *
+const char *
 pgtype_to_name(StatementClass *stmt, Int4 type)
 {
 	ConnectionClass	*conn = SC_get_conn(stmt);
@@ -527,7 +530,7 @@ pgtype_to_name(StatementClass *stmt, Int4 type)
 		case PG_TYPE_BYTEA:
 			return "bytea";
 
-		case PG_TYPE_LO:
+		case PG_TYPE_LO_UNDEFINED:
 			return PG_TYPE_LO_NAME;
 
 		default:
@@ -815,6 +818,8 @@ getTimestampColumnSize(StatementClass *stmt, Int4 type, int col)
 Int4
 pgtype_column_size(StatementClass *stmt, Int4 type, int col, int handle_unknown_size_as)
 {
+	ConnectionClass *conn = SC_get_conn(stmt);
+
 	switch (type)
 	{
 		case PG_TYPE_CHAR:
@@ -828,7 +833,6 @@ pgtype_column_size(StatementClass *stmt, Int4 type, int col, int handle_unknown_
 
 		case PG_TYPE_NAME:
 			{
-				ConnectionClass *conn = SC_get_conn(stmt);
 				int	value = 0;
 #ifdef	NAME_FIELD_SIZE
 				value = NAME_FIELD_SIZE;
@@ -878,12 +882,9 @@ pgtype_column_size(StatementClass *stmt, Int4 type, int col, int handle_unknown_
 			return getTimestampColumnSize(stmt, type, col);
 
 		case PG_TYPE_BOOL:
-		{
-			BOOL	true_is_minus1 = FALSE;
-			return true_is_minus1 ? 2 : 1;
-		}
+			return conn->connInfo.true_is_minus1 ? 2 : 1;
 
-		case PG_TYPE_LO:
+		case PG_TYPE_LO_UNDEFINED:
 			return SQL_NO_TOTAL;
 
 		default:
@@ -898,7 +899,7 @@ pgtype_column_size(StatementClass *stmt, Int4 type, int col, int handle_unknown_
 }
 
 /*
- *	"precision in ODBC 3.x.
+ *	precision in ODBC 3.x.
  */
 Int4
 pgtype_precision(StatementClass *stmt, Int4 type, int col, int handle_unknown_size_as)
@@ -1355,7 +1356,7 @@ pgtype_create_params(StatementClass *stmt, Int4 type)
 
 
 Int2
-sqltype_to_default_ctype(Int2 sqltype)
+sqltype_to_default_ctype(const ConnectionClass *conn, Int2 sqltype)
 {
 	/*
 	 * from the table on page 623 of ODBC 2.0 Programmer's Reference
@@ -1381,6 +1382,8 @@ sqltype_to_default_ctype(Int2 sqltype)
 		case SQL_WCHAR:
 		case SQL_WVARCHAR:
 		case SQL_WLONGVARCHAR:
+			if (conn->ms_jet || ! conn->unicode)
+				return SQL_C_CHAR;
 			return SQL_C_WCHAR;
 #endif /* UNICODE_SUPPORT */
 
