@@ -154,6 +154,8 @@ PGAPI_FreeStmt(HSTMT hstmt,
 		/* Remove the statement from the connection's statement list */
 		if (conn)
 		{
+			QResultClass	*res;
+
 			if (!CC_remove_statement(conn, stmt))
 			{
 				SC_set_error(stmt, STMT_SEQUENCE_ERROR, "Statement is currently executing a transaction.");
@@ -163,9 +165,9 @@ PGAPI_FreeStmt(HSTMT hstmt,
 			}
 
 			/* Free any cursors and discard any result info */
-			if (SC_get_Result(stmt))
+			if (res = SC_get_Result(stmt), res)
 			{
-				QR_Destructor(SC_get_Result(stmt));
+				QR_Destructor(res);
 				SC_set_Result(stmt,  NULL);
 			}
 		}
@@ -254,6 +256,7 @@ SC_Constructor(void)
 		rv->curres = NULL;
 		rv->manual_result = FALSE;
 		rv->prepare = FALSE;
+		rv->prepared = FALSE;
 		rv->status = STMT_ALLOCATED;
 		rv->internal = FALSE;
 
@@ -264,6 +267,7 @@ SC_Constructor(void)
 		rv->statement = NULL;
 		rv->stmt_with_params = NULL;
 		rv->load_statement = NULL;
+		rv->execute_statement = NULL;
 		rv->stmt_size_limit = -1;
 		rv->statement_type = STMT_TYPE_UNKNOWN;
 
@@ -302,6 +306,7 @@ SC_Constructor(void)
 		rv->updatable = FALSE;
 		rv->error_recsize = -1;
 		rv->diag_row_count = 0;
+		rv->stmt_time = 0;
 		INIT_STMT_CS(rv);
 	}
 	return rv;
@@ -352,6 +357,8 @@ void IRDFields_free(IRDFields * self)
 
 void IPDFields_free(IPDFields * self)
 {
+	/* param bindings */
+	IPD_free_params(self, STMT_FREE_PARAMS_ALL);
 }
 
 char
@@ -359,6 +366,7 @@ SC_Destructor(StatementClass *self)
 {
 	QResultClass	*res = SC_get_Result(self);
 
+	if (!self)	return FALSE;
 	mylog("SC_Destructor: self=%u, self->result=%u, self->hdbc=%u\n", self, res, self->hdbc);
 	SC_clear_error(self);
 	if (STMT_EXECUTING == self->status)
@@ -375,15 +383,7 @@ SC_Destructor(StatementClass *self)
 		QR_Destructor(res);
 	}
 
-	if (self->statement)
-		free(self->statement);
-	if (self->stmt_with_params)
-	{
-		free(self->stmt_with_params);
-		self->stmt_with_params = NULL;
-	}
-	if (self->load_statement)
-		free(self->load_statement);
+	SC_initialize_stmts(self, TRUE);
 
         /* Free the parsed table information */
 	if (self->ti)
@@ -423,6 +423,7 @@ void
 SC_free_params(StatementClass *self, char option)
 {
 	APD_free_params(SC_get_APD(self), option);
+	IPD_free_params(SC_get_IPD(self), option);
 	self->data_at_exec = -1;
 	self->current_exec_param = -1;
 	self->put_data = FALSE;
@@ -436,7 +437,7 @@ SC_free_params(StatementClass *self, char option)
 
 
 int
-statement_type(char *statement)
+statement_type(const char *statement)
 {
 	int			i;
 
@@ -451,6 +452,63 @@ statement_type(char *statement)
 	return STMT_TYPE_OTHER;
 }
 
+void
+SC_set_prepared(StatementClass *stmt, BOOL prepared)
+{
+	if (prepared == stmt->prepared)
+		return;
+	if (!prepared)
+	{
+		ConnectionClass *conn = SC_get_conn(stmt);
+		if (CONN_CONNECTED == conn->status)
+		{
+			QResultClass	*res;
+			char dealloc_stmt[128];
+
+			sprintf(dealloc_stmt, "DEALLOCATE _PLAN%0x", stmt);
+			res = CC_send_query(conn, dealloc_stmt, NULL, 0);
+			if (res)
+				QR_Destructor(res); 
+		}
+	}
+	stmt->prepared = prepared;
+}
+
+/*
+ *	Initialize stmt_with_params, load_statement and execute_statement
+ *		member pointer deallocating corresponding prepared plan.
+ *	Also initialize statement member pointer if specified.
+ */
+RETCODE
+SC_initialize_stmts(StatementClass *self, BOOL initializeOriginal)
+{
+	if (initializeOriginal)
+	{
+		if (self->statement)
+		{
+			free(self->statement);
+			self->statement = NULL;
+		}
+		if (self->execute_statement)
+		{
+			free(self->execute_statement);
+			self->execute_statement = NULL;
+		}
+		SC_set_prepared(self,FALSE);
+	}
+	if (self->stmt_with_params)
+	{
+		free(self->stmt_with_params);
+		self->stmt_with_params = NULL;
+	}
+	if (self->load_statement)
+	{
+		free(self->load_statement);
+		self->load_statement = NULL;
+	}
+
+	return 0;
+}
 
 /*
  *	Called from SQLPrepare if STMT_PREMATURE, or
@@ -554,12 +612,7 @@ SC_recycle_statement(StatementClass *self)
 	 * SQLParamData/SQLPutData is called.
 	 */
 	SC_free_params(self, STMT_FREE_PARAMS_DATA_AT_EXEC_ONLY);
-	if (self->stmt_with_params)
-		free(self->stmt_with_params);
-	self->stmt_with_params = NULL;
-	if (self->load_statement)
-		free(self->load_statement);
-	self->load_statement = NULL;
+	SC_initialize_stmts(self, FALSE);
 	/*
 	 *	reset the current attr setting to the original one.
 	 */
@@ -763,6 +816,15 @@ SC_get_error(StatementClass *self, int *number, char **message)
 }
 
 
+time_t
+SC_get_time(StatementClass *stmt)
+{
+	if (!stmt)
+		return time(NULL);
+	if (!stmt->stmt_time)
+		stmt->stmt_time = time(NULL);
+	return stmt->stmt_time;
+}
 /*
  *	Currently, the driver offers very simple bookmark support -- it is
  *	just the current row number.  But it could be more sophisticated
@@ -1101,8 +1163,13 @@ SC_execute(StatementClass *self)
 		}
 		else
 		{
+			QResultClass	*tres;
+
 			/* see if the query did return any result columns */
-			numcols = QR_NumResultCols(res);
+			for (tres = res, numcols = 0; !numcols && tres; tres = tres->next)
+			{
+				numcols = QR_NumResultCols(tres);
+			}
 			/* now allocate the array to hold the binding info */
 			if (numcols > 0)
 			{
